/////////////////////////////////////////////////////////////////////////////
// Name:        pendlgg.cpp
// Purpose:     Choose pen
// Author:      Martin Bernreuther
// Modified by:
// Created:     29/07/02
// Licence:     wxWindows license
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
#pragma implementation "pendlgg.h"
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/dialog.h"
#include "wx/stattext.h"
#include "wx/choice.h"
//#include "wx/button.h"
#include "wx/bmpbuttn.h"
#include "wx/layout.h"
#include "wx/sizer.h"
#include "wx/dcclient.h"
#include "wx/dcmemory.h"
#endif

#include "wx/spinctrl.h"
#include "wx/colordlg.h"

#if wxUSE_STATLINE
    #include "wx/statline.h"
#endif

#include "wx/generic/pendlgg.h"



/*
 * Generic wxPenDialog
 */


wxGenPenDlgView::wxGenPenDlgView(wxWindow *parent, const wxPen *pen)
		: wxWindow(parent,-1, wxDefaultPosition, wxSize(120,20),wxSUNKEN_BORDER), m_pen(pen)
{
	Connect( -1, wxEVT_PAINT, (wxObjectEventFunction) &wxGenPenDlgView::OnPaint );
}

void wxGenPenDlgView::OnPaint(wxPaintEvent& event)
{
  wxPaintDC dc(this);

  wxCoord w,h;
  GetClientSize(&w,&h);
  wxPoint points[4];
  points[0].x=int(.75*w);
  points[0].y=int(.75*h);
  points[1].x=points[2].x=int(.1*w);
  points[3].x=int(.9*w);
  points[1].y=int(.1*h);
  points[2].y=points[3].y=int(.5*h);

  dc.BeginDrawing();
  dc.Clear();

  dc.SetPen(*m_pen);
  //dc.SetLogicalFunction(wxCOPY);
  //dc.DrawLine(int(.1*w),int(.5*h),int(.9*w),int(.5*h));
  dc.DrawLines(4,points);

  dc.EndDrawing();
}

// ----------------------------------------------------------------------------


#define wxID_STYLECHOICE   3000
#define wxID_WIDTHSPINCTRL 3001
#define wxID_COLOURBUTTON  3002
#define wxID_CAPCHOICE     3003
#define wxID_JOINCHOICE    3004

wxString wxGenericPenDialog::PenStyleNames[]={
            wxT("SOLID"),
            wxT("DOT"),
            wxT("LONG_DASH"),
            wxT("SHORT_DASH"),
            wxT("DOT_DASH"),
            //wxT("USER_DASH"),
            wxT("TRANSPARENT"),
            //wxT("STIPPLE"),
            wxT("BDIAGONAL_HATCH"),
            wxT("CROSSDIAG_HATCH"),
            wxT("FDIAGONAL_HATCH"),
            wxT("CROSS_HATCH"),
            wxT("HORIZONTAL_HATCH"),
            wxT("VERTICAL_HATCH")
            };
wxString wxGenericPenDialog::PenCapNames[]={
            wxT("ROUND"),
            wxT("PROJECTING"),
            wxT("BUTT")
            };
wxString wxGenericPenDialog::PenJoinNames[]={
            wxT("ROUND"),
            wxT("BEVEL"),
            wxT("MITER")
            };
wxString wxGenericPenDialog::Labels[]={
            wxT("Style"),
            wxT("Width"),
            wxT("Colour"),
            wxT("Cap"),
            wxT("Join")
            };
	


wxGenericPenDialog::wxGenericPenDialog(wxWindow *parent,const wxPen *pen, bool showJoin)
{
  m_stylechoice=(wxChoice*)NULL;
  m_widthspinctrl=(wxSpinCtrl*)NULL;
  //m_colourbutton=(wxButton*)NULL;
  m_colourbutton=(wxBitmapButton*)NULL;
  m_capchoice=(wxChoice*)NULL;
  m_joinchoice=(wxChoice*)NULL;
  m_view=(wxGenPenDlgView*)NULL;

  Create(pen,parent,showJoin);
}

void wxGenericPenDialog::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
  EndModal(wxID_CANCEL);
}

bool wxGenericPenDialog::Create(const wxPen *pen, wxWindow *parent, bool showJoin)
{
    if ( !wxDialog::Create(parent, -1, wxT("Choose Pen"),
                           wxPoint(0, 0), wxSize(900, 900),
                           wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL) )
        return FALSE;

    m_parent = parent;

    if (!pen || *pen==wxNullPen)
        m_pen = *wxBLACK_PEN;
    else
        m_pen = *pen;

    CreateWidgets(showJoin);

    Connect(wxID_STYLECHOICE,wxEVT_COMMAND_CHOICE_SELECTED,(wxObjectEventFunction) &wxGenericPenDialog::OnStyleChoice);
    Connect(wxID_WIDTHSPINCTRL,wxEVT_COMMAND_SPINCTRL_UPDATED,(wxObjectEventFunction) &wxGenericPenDialog::OnWidthSpinCtrl);
    Connect(wxID_COLOURBUTTON,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction) &wxGenericPenDialog::OnColourButtonClick);
    Connect(wxID_CAPCHOICE,wxEVT_COMMAND_CHOICE_SELECTED,(wxObjectEventFunction) &wxGenericPenDialog::OnCapChoice);
    if(showJoin)
		Connect(wxID_JOINCHOICE,wxEVT_COMMAND_CHOICE_SELECTED,(wxObjectEventFunction) &wxGenericPenDialog::OnJoinChoice);

    return TRUE;
}

int wxGenericPenDialog::ShowModal()
{
  return wxDialog::ShowModal();
}


// Internal functions
void wxGenericPenDialog::CreateWidgets(bool showJoin)
{
    wxBeginBusyCursor();

    int w,h;
    int maxwidth, totalwidth;

    wxStaticText *stylelabel = new wxStaticText(this,-1,Labels[0]);
    stylelabel->GetSize(&w,&h);
    maxwidth=w;
    wxStaticText *widthlabel = new wxStaticText(this,-1,Labels[1]);
    widthlabel->GetSize(&w,&h);
    if(w>maxwidth) maxwidth=w;
    wxStaticText *colourlabel = new wxStaticText(this,-1,Labels[2]);
    colourlabel->GetSize(&w,&h);
    if(w>maxwidth) maxwidth=w;
    wxStaticText *caplabel = new wxStaticText(this,-1,Labels[3]);
    caplabel->GetSize(&w,&h);
    if(w>maxwidth) maxwidth=w;
	wxStaticText *joinlabel;
	if(showJoin)
	{
      joinlabel = new wxStaticText(this,-1,Labels[4]);
      joinlabel->GetSize(&w,&h);
      if(w>maxwidth) maxwidth=w;
	}

    stylelabel->SetSize(maxwidth,-1);
    widthlabel->SetSize(maxwidth,-1);
    colourlabel->SetSize(maxwidth,-1);
    caplabel->SetSize(maxwidth,-1);
	if(showJoin) joinlabel->SetSize(maxwidth,-1);
    totalwidth=maxwidth;

    m_stylechoice = new wxChoice(this,wxID_STYLECHOICE,wxDefaultPosition,wxDefaultSize,12,PenStyleNames);
    m_stylechoice->GetSize(&w,&h);
    switch(m_pen.GetStyle())
    {
    case wxSOLID: m_stylechoice->SetSelection(0); break;
    case wxDOT: m_stylechoice->SetSelection(1); break;
    case wxLONG_DASH: m_stylechoice->SetSelection(2); break;
    case wxSHORT_DASH: m_stylechoice->SetSelection(3); break;
    case wxDOT_DASH: m_stylechoice->SetSelection(4); break;
    case wxTRANSPARENT: m_stylechoice->SetSelection(5); break;
    case wxBDIAGONAL_HATCH: m_stylechoice->SetSelection(6); break;
    case wxCROSSDIAG_HATCH: m_stylechoice->SetSelection(7); break;
    case wxFDIAGONAL_HATCH: m_stylechoice->SetSelection(8); break;
    case wxCROSS_HATCH: m_stylechoice->SetSelection(9); break;
    case wxHORIZONTAL_HATCH: m_stylechoice->SetSelection(10); break;
    case wxVERTICAL_HATCH: m_stylechoice->SetSelection(11); break;
    //case wxUSER_DASH: m_stylechoice->SetSelection(12); break;
    //case wxSTIPPLE: m_stylechoice->SetSelection(13); break;
    }
    maxwidth=w;
    m_widthspinctrl = new wxSpinCtrl(this,wxID_WIDTHSPINCTRL,"Width");
    m_widthspinctrl->GetSize(&w,&h);
    m_widthspinctrl->SetRange(0,25);
    m_widthspinctrl->SetValue(m_pen.GetWidth());
    if(w>maxwidth) maxwidth=w;

    //m_colourbutton = new wxButton(this,wxID_COLOURBUTTON,"");
    //m_colourbutton->SetBackgroundColour(m_pen.GetColour());
    wxBitmap bitmap(maxwidth,h);
    wxMemoryDC dc;
    dc.SelectObject(bitmap);
    dc.SetBackground(wxBrush(m_pen.GetColour(),wxSOLID));
    dc.Clear();
    dc.SelectObject(wxNullBitmap);
    m_colourbutton = new wxBitmapButton(this,wxID_COLOURBUTTON,bitmap);

    m_colourbutton->GetSize(&w,&h);
    if(w>maxwidth) maxwidth=w;
    m_capchoice = new wxChoice(this,wxID_CAPCHOICE,wxDefaultPosition,wxDefaultSize,3,PenCapNames);
    m_capchoice->GetSize(&w,&h);
    switch(m_pen.GetCap())
    {
    case wxCAP_ROUND: m_capchoice->SetSelection(0); break;
    case wxCAP_PROJECTING: m_capchoice->SetSelection(1); break;
    case wxCAP_BUTT: m_capchoice->SetSelection(2); break;
    }
    if(w>maxwidth) maxwidth=w;
	if(showJoin)
	{
      m_joinchoice = new wxChoice(this,wxID_JOINCHOICE,wxDefaultPosition,wxDefaultSize,3,PenJoinNames);
      m_joinchoice->GetSize(&w,&h);
      switch(m_pen.GetJoin())
	  {
      case wxJOIN_ROUND: m_joinchoice->SetSelection(0); break;
      case wxJOIN_BEVEL: m_joinchoice->SetSelection(1); break;
      case wxJOIN_MITER: m_joinchoice->SetSelection(2); break;
	  }
      if(w>maxwidth) maxwidth=w;
	}

    m_stylechoice->SetSize(maxwidth,-1);
    m_widthspinctrl->SetSize(maxwidth,-1);
    m_colourbutton->SetSize(maxwidth,-1);
    m_capchoice->SetSize(maxwidth,-1);
	if(showJoin) m_joinchoice->SetSize(maxwidth,-1);
    totalwidth+=10+maxwidth;

    wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *horsizer = new wxBoxSizer(wxHORIZONTAL);
    horsizer->Add(stylelabel,0,wxALIGN_CENTER|wxALL,5);
    horsizer->Add(m_stylechoice,1,wxALIGN_CENTER|wxALL,5);
    topsizer->Add(horsizer,0,wxALIGN_CENTER|wxALL,5);

    horsizer = new wxBoxSizer(wxHORIZONTAL);
    horsizer->Add(widthlabel,0,wxALIGN_CENTER|wxALL,5);
    horsizer->Add(m_widthspinctrl,1,wxALIGN_CENTER|wxALL,5);
    topsizer->Add(horsizer,0,wxALIGN_CENTER|wxALL,5);

    horsizer = new wxBoxSizer(wxHORIZONTAL);
    horsizer->Add(colourlabel,0,wxALIGN_CENTER|wxALL,5);
    horsizer->Add(m_colourbutton,1,wxALIGN_CENTER|wxALL,5);
    topsizer->Add(horsizer,0,wxALIGN_CENTER|wxALL,5);

    horsizer = new wxBoxSizer(wxHORIZONTAL);
    horsizer->Add(caplabel,0,wxALIGN_CENTER|wxALL,5);
    horsizer->Add(m_capchoice,1,wxALIGN_CENTER|wxALL,5);
    topsizer->Add(horsizer,0,wxALIGN_CENTER|wxALL,5);

	if(showJoin)
	{
      horsizer = new wxBoxSizer(wxHORIZONTAL);
      horsizer->Add(joinlabel,0,wxALIGN_CENTER|wxALL,5);
      horsizer->Add(m_joinchoice,1,wxALIGN_CENTER|wxALL,5);
      topsizer->Add(horsizer,0,wxALIGN_CENTER|wxALL,5);
	}

    m_view = new wxGenPenDlgView(this,&m_pen);
    m_view->SetSize(totalwidth,50);
    m_view->SetBackgroundColour(*wxWHITE);
    topsizer->Add(m_view,1,wxALIGN_CENTER|wxALL,5);
    
#if wxUSE_STATLINE
    // static line
    topsizer->Add( new wxStaticLine( this, -1 ), 0, wxEXPAND | wxLEFT|wxRIGHT|wxTOP, 10 );
#endif

    // OK/Cancel buttons
    wxSizer *buttonsizer = CreateButtonSizer( wxOK|wxCANCEL );
    topsizer->Add( buttonsizer, 0, wxCENTRE | wxALL, 10 );

    SetAutoLayout( TRUE );
    SetSizer( topsizer );
    
    topsizer->SetSizeHints( this );
    topsizer->Fit( this );

    Centre( wxBOTH );

    Refresh();

    wxEndBusyCursor();
}


void wxGenericPenDialog::OnStyleChoice()
{
    switch(m_stylechoice->GetSelection())
    {
    case 0: m_pen.SetStyle(wxSOLID); break;
    case 1: m_pen.SetStyle(wxDOT); break;
    case 2: m_pen.SetStyle(wxLONG_DASH); break;
    case 3: m_pen.SetStyle(wxSHORT_DASH); break;
    case 4: m_pen.SetStyle(wxDOT_DASH); break;
    case 5: m_pen.SetStyle(wxTRANSPARENT); break;
    case 6: m_pen.SetStyle(wxBDIAGONAL_HATCH); break;
    case 7: m_pen.SetStyle(wxCROSSDIAG_HATCH); break;
    case 8: m_pen.SetStyle(wxFDIAGONAL_HATCH); break;
    case 9: m_pen.SetStyle(wxCROSS_HATCH); break;
    case 10: m_pen.SetStyle(wxHORIZONTAL_HATCH); break;
    case 11: m_pen.SetStyle(wxVERTICAL_HATCH); break;
    //case 12: m_pen.SetStyle(wxUSER_DASH); break;
    //case 13: m_pen.SetStyle(wxSTIPPLE); break;
    }
    if(m_stylechoice->GetSelection()>5 && m_pen.GetWidth()<2)
    {
	m_pen.SetWidth(2);
	m_widthspinctrl->SetValue(2);
    }
    m_view->Refresh();
}

void wxGenericPenDialog::OnWidthSpinCtrl()
{
    m_pen.SetWidth(m_widthspinctrl->GetValue());
    m_view->Refresh();
}

void wxGenericPenDialog::OnColourButtonClick()
{
    wxColourDialog *dialog = new wxColourDialog(this,&m_colourdata);
    dialog->SetTitle("Choose pen colour");
    if (dialog->ShowModal() == wxID_OK)
    {
        wxColourData colour = dialog->GetColourData();
        wxColour col = colour.GetColour();
	    m_pen.SetColour(col);
		m_colourdata=dialog->GetColourData();

        //m_colourbutton->SetBackgroundColour(col);
	    wxMemoryDC dc;
	    dc.SelectObject(m_colourbutton->GetBitmapLabel());
	    dc.SetBackground(wxBrush(col,wxSOLID));
	    dc.Clear();
	    dc.SelectObject(wxNullBitmap);

        m_view->Refresh();
    }
    dialog->Destroy();
}

void wxGenericPenDialog::OnCapChoice()
{
    switch(m_capchoice->GetSelection())
    {
    case 0: m_pen.SetCap(wxCAP_ROUND); break;
    case 1: m_pen.SetCap(wxCAP_PROJECTING); break;
    case 2: m_pen.SetCap(wxCAP_BUTT); break;
    }
    m_view->Refresh();
}

void wxGenericPenDialog::OnJoinChoice()
{
    switch(m_joinchoice->GetSelection())
    {
    case 0: m_pen.SetJoin(wxJOIN_ROUND); break;
    case 1: m_pen.SetJoin(wxJOIN_BEVEL); break;
    case 2: m_pen.SetJoin(wxJOIN_MITER); break;
    }
    m_view->Refresh();
}



// ----------------------------------------------------------------------------


bool WXDLLEXPORT wxGetPenFromUser(wxPen& pen
                                 ,const wxString& title, bool showJoin
								 ,wxColourData* colourdata
                                 ,wxWindow *parent)
{
	wxGenericPenDialog *dialog=new wxGenericPenDialog(NULL,&pen,showJoin);
	dialog->SetTitle(title);
	if(colourdata) dialog->SetColourData(*colourdata);
	if(dialog->ShowModal()==wxID_OK)
	{
		pen=dialog->GetPen();
		if(colourdata) *colourdata=dialog->GetColourData();
		dialog->Destroy();
		return TRUE;
	}
	else
	{
		dialog->Destroy();
		return FALSE;
	}
}
